      PROGRAM GEOMCORR
C** *****************************************************************
C
C    REMAPP P-E/3220 1980
C   REMOTE SENSING ARRAY PROCESSING PROCEDURES
C   U. S. GEOLOGICAL SURVEY, DENVER, COLORADO
C   BRANCH OF PETROPHYSICS AND REMOTE SENSING
C   DON L. SAWATZKY
C
C   *****************************************************************
C
C
C     PURPOSE
C       GEOMETRIC RECTIFICATION FOR LANDSAT MSS IMAGERY IS
C       PERFORMED ON A LINE BY LINE BASIS FOR OVER-SAMPLING
C       IN THE SCAN LINE DIRECTION (ASPECT RATIO
C       CORRECTION), AND THE NON-POLAR ORBIT OF THE
C       SATELLITE AND THE ROTATION OF THE EARTH
C       UNDER THE SATELLITE (SKEW CORRECTION).
C
C
C     DESCRIPTION OF PARAMETERS
C       LATD   = DEGREES OF LATITUDE,  {NW CORNER OF FRAME}
C       LATM   = MINUTES OF LATITUDE
C       LATS   = SECONDS OF LATITUDE
C       PIXELS = ORIGINAL NUMBER OF PIXELS PER SCAN LINE
C       NL     = NUMBER OF LINES IN THE IMAGE
C       KOUNT  = PROGRAM CALL COUNTER
C       NSGM   = CORRECTED NUMBER OF PIXELS PER LINE
C       IBUF   = BUFFER CONTAINING LINE TO BE CORRECTED
C
C       ALL PARAMETERS ARE INTEGERS GREATER THAN OR EQUAL
C       TO ZERO.
C
C     SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C       FILBUF
C       COS
C
C     LINK WITH DISKIO, REMLOG, DATE, TIME, EXIT
C
C     METHOD
C       SKEWING OF THE IMAGE IS PERFORMED BY OFFSETTING
C       THE SCAN LINES AN INTEGER AMOUNT DETERMINED
C       FROM THE LATITUDE. ASPECT RATIO IS CORRECTED BY
C       DELETING PIXELS FROM THE LINE.
C
C     PROGRAM FROM FASTGE.FTN SUBROUTINE (of tapeio)
C
C     ORIGINALLY CODED BY. .
C       PAT CHAVEZ
C       U.S. GEOLOGICAL SURVEY
C       FLAGSTAFF, ARIZONA
C
C     MODIFICATIONS BY. .
C       TIMOTHY E. TOWNSEND
C       U.S. GEOLOGICAL SURVEY
C       DENVER, COLORADO
C
      INTEGER*4 LATD,LATM,LATS,PIXELS,NL,KOUNT,NSGM
      INTEGER*4 ISPOUT(950), FCB(128), FCBO(128)
      LOGICAL*1 IBUF(4096),LDN/.FALSE./
      INTEGER*4 TTYOUT,NS
      EQUIVALENCE(NS,PIXELS)
      DATA TTYOUT/6/
C  OPEN INPUT FILE
      CALL REMLOG('***   MSS GEOMCORR   ***')
      CALL DISKIO(0,12,IBUF,FCB)
      IF(FCB(1).LT.0) CALL EXIT(3)
      FCBO(1)=8
      FCBO(2)=FCB(2)
      CALL DISKIO(0,13,IBUF,FCBO)
      IF(FCB(1).LT.0) CALL EXIT(3)
C  INITIALIZE VARIABLES
      PIXELS=FCB(2)
      NL=FCB(3)
      KOUNT=1
      NSGM=FCB(2)
      WRITE(6,*) 'LATITUDE in DEGREES, MINUTES, SECONDS: '
      READ(5,*) LATD, LATM, LATS
C
       ANL1=1-NL
       ANL=NL
       ALAT=LATS/60.0
       ALAT=ALAT+LATM
       ALAT=ALAT/60.0
       ALAT=ALAT+LATD
       IDIFF=PIXELS-NL
       DIFF=IDIFF
       TOLSP=PIXELS
C
C  DELTA=INCREMENT WHICH TELLS THE PROGRAM WHAT SAMPLES
C        TO THROW OUT FOR THE ASPECT RATIO CORRECTION.
C
C        NS=3240
C        NS-NL=3240-2340=900
C
       DELTA=3240.0/900.0
       APRATO=2340.0/3240.0
       ANSGM=NS*APRATO
       NSGM2=ANSGM
       VELEQ=465.1
       PI=3.14159
       ALAT=ALAT*(PI/180.0)
       VELAT=VELEQ*COS(ALAT)
       TIMET=28.6338
       TIME1=0.07342
       ANL=NL
       SCANNO=ANL/6.0
       TIMENL=SCANNO*TIME1
       DELTAM=TIMENL*VELAT
       AREAPX=(185000.0/2340.0)
       SKEWPX=DELTAM/AREAPX+0.50
C
       CMPFAC=1.0
       SKEWPX=SKEWPX/CMPFAC+0.50
       ISKEW=SKEWPX
       AISKEW=ISKEW
       NSGM=ANSGM+ISKEW
       NSGM1=ANSGM
C
C  NON-POLAR ORBIT
C
C  BUILD A TABLE WHICH HAS THE NUMBER OF THE SAMPLE
C  TO BE ELIMINATED.
C
       NSOUT=950
       DO 100 I=1,NSOUT
          AI=I
          PTOUT=DELTA*AI
          ISPOUT(I)=PTOUT+0.50
100       CONTINUE
C
       WRITE(4,110)ISKEW,NSGM1,NSGM,LATD,LATM,LATS
110       FORMAT(/1X,T15,'ERTS GEOMETRIC RECTIFICATION ',
     *   'PERFORMED:',/,1X,T15,'THE AMOUNT OF SKEW IS ',I5,
     *   ' PIXELS.',/,1X,T15,'THE NUMBER OF PIXELS IN THE ',
     *   'ACTUAL IMAGE AREA IS ',I5,'.',/1X,T15,
     *   'THE TOTAL NUMBER OF PIXELS IS ',I5,'.',
     *   /1X,T15,'LATITUDE OF NW CORNER = ',I3,'D',I3,'M',I3,'S')
C
C  ASPECT RATIO CORRECTION  (COMPRESSION)
C
       KOUNT4=0
       AIBLKL=(PIXELS+2047.0)/2048.0
       IBLKL=AIBLKL
C
C  START READ SCANLINE LOOP
C
900    KOUNT=KOUNT+1
       CALL DISKIO(9,12,IBUF,FCB)
       IF(FCB(1).LT.0) GOTO 999
       INDEX=1
       INDEX2=0
       KOUNT4=KOUNT4+1
       IF(KOUNT4.EQ.IBLKL)KOUNT4=0
C
       IS=0
130       IS=IS+1
       IF(IS.GT.NS)GO TO 160
       INDEX2=INDEX2+1
       IF(IS.LT.ISPOUT(INDEX))GO TO 140
       IS1=IS+1
C      IBUF(INDEX2)=(IBUF(IS)+IBUF(IS1))/2
      IBUF(INDEX2)=IBUF(IS)
       IS=IS+1
       INDEX=INDEX+1
       GO TO 150
140       IBUF(INDEX2)=IBUF(IS)
150       GO TO 130
160       CONTINUE
C
C  LSKEW=NUMBER OF PIXELS TO SKEW THIS LINE.
C
       AKOUNT=KOUNT
       SKEW=(AKOUNT-ANL)/ANL1*AISKEW
C
       LSKEW=SKEW
       IF(LSKEW.LT.1)GO TO 170
       CALL FILBUF(IBUF(LSKEW),IBUF(1),-NSGM1)
C
C  ZERO OUT THE FIRST PART OF ARRAY, AND END.
C
       LSKEW1=LSKEW-1
       IBUF(1)=LDN
       IF(LSKEW1.LT.1)GO TO 170
       CALL FILBUF(IBUF(2),IBUF(1),LSKEW1)
170       NSPTS=ISKEW-LSKEW
       NSEND=NSGM-NSPTS
       IBUF(NSEND)=LDN
       NSEND1=NSEND+1
C
C  USE COMMENTED STATMENT BELOW IF WANT TO TRUCATE FILE
C    TO USABLE DATA ONLY  (OR NEXT STATEMENT AFTER - TO
C    APPLY DN=0 FILL FOR SPACING TO ORGINAL RECORD LENGTH)
C
C       NSPTS=NSPTS-1
       NSPTS=FCBO(2) - NSEND
C
       IF(NSPTS.LT.1)GO TO 180
       CALL FILBUF(IBUF(NSEND1),IBUF(NSEND),NSPTS)
180    CALL DISKIO(10,13,IBUF,FCBO)
       IF(FCB(1).LT.0) GOTO 999
       GOTO 900
C
C  CLOSE FILES
C
999    CALL DISKIO(6,12,IBUF,FCB)
       CALL DISKIO(6,13,IBUF,FCBO)
       STOP
       END
C
C
      SUBROUTINE FILBUF(OUT,IN,N)
      LOGICAL*1 IN(1),OUT(1)
      IF(N) 30,10,20
10    RETURN
20    I=0
21    I=I+1
      OUT(I)=IN(I)
      IF(I.LT.N) GOTO 21
      RETURN
30    DO 31 I=N,-1
31    OUT(-I)=IN(-I)
      RETURN
      END
